unit Main;

interface

{$WARN UNIT_PLATFORM OFF}

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, FileCtrl,
  Dialogs, ExtCtrls, StdCtrls, ComCtrls, OleCtrls, SHDocVw,
  ImgList, ToolWin, Buttons,
  //
  stdStrings, stdVCLCtrl, stdInt, StrToken, stdFiles, ExIniFiles;

type
  TJManualViewerMainForm = class(TForm)
    APISearchBox: TGroupBox;
    PackageList: TComboBox;
    QuickSearch: TEdit;
    ClassesList: TListBox;
    Splitter: TSplitter;
    BrowsingPanel: TPanel;
    StatusBar: TStatusBar;
    ImageList: TImageList;
    TBPanel: TPanel;
    ReloadPage: TSpeedButton;
    ToTopPage: TSpeedButton;
    SetHelpPath: TSpeedButton;
    URLCombo: TComboBox;
    Browser: TWebBrowser;
    procedure FormCreate(Sender: TObject);
    procedure BrowserStatusTextChange(Sender: TObject;
      const Text: WideString);
    procedure PackageListClick(Sender: TObject);
    procedure ClassesListDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure FormDestroy(Sender: TObject);
    procedure QuickSearchChange(Sender: TObject);
    procedure ClassesListDblClick(Sender: TObject);
    procedure ClassesListKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure BrowserTitleChange(Sender: TObject; const Text: WideString);
    procedure BrowserBeforeNavigate2(Sender: TObject;
      const pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
      Headers: OleVariant; var Cancel: WordBool);
    procedure ReloadPageClick(Sender: TObject);
    procedure ToTopPageClick(Sender: TObject);
    procedure SetHelpPathClick(Sender: TObject);
    procedure BrowserNavigateComplete2(Sender: TObject;
      const pDisp: IDispatch; var URL: OleVariant);
    procedure URLComboClick(Sender: TObject);
    procedure URLComboKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure QuickSearchKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormShortCut(var Msg: TWMKey; var Handled: Boolean);
  private
    { Private 錾 }
    APIManualRoot : String;
    ClassesItem: TStringList;
    procedure ReloadAPIList();
    procedure GoPage(RelativeURL: String);
    function  PackageNameToPath(): String;
  public
    { Public 錾 }
  end;

var
  JManualViewerMainForm: TJManualViewerMainForm;

const
  INTERFACE_ICON = 'i';
  CLASS_ICON = 'c';
  EXCEPTION_ICON = 'e';
implementation

uses Math, StrUtils;

{$R *.dfm}

{ TJManualViewerMainForm }

procedure TJManualViewerMainForm.FormCreate(Sender: TObject);
  procedure SetFind(AClassName: String);
  var
    cn: Integer;
  label next;
  begin
    for cn := 0 to ClassesItem.Count - 1 do
      if AnsiContainsText(PickConstTokenRev(ClassesItem[cn], '/'), AClassName + '.html') then
        goto next;
    ToTopPage.Click;
    exit;
  next:
    ClassesList.ItemIndex := cn;
    ClassesListDblClick(nil);
  end;

begin
  // iniǂݍ
  with TMemIniFileEx.CreateDef do begin try
    Section           := 'General';
    APIManualRoot     := ReadStringS('Manual', '');
    BoundsRect        := ReadRectS('Size', BoundsRect);
    APISearchBox.Width:= ReadIntegerS('LeftWidth', APISearchBox.Width);
  finally
    Free;
  end;
  end;

  Caption := Application.Title;
  ClassesItem := TStringList.Create;
  ReloadAPIList;
  if ParamCount = 1 then
    SetFind(ParamStr(1))
  else
    ToTopPage.Click;
end;

procedure TJManualViewerMainForm.FormDestroy(Sender: TObject);
begin
  // ini
  with TMemIniFileEx.CreateDef do begin try
    Section := 'General';
    WriteStringS('Manual', APIManualRoot);
    WindowState := wsNormal;
    WriteRectS('Size', BoundsRect);
    WriteIntegerS('LeftWidth', APISearchBox.Width);
    UpdateFile;
  finally
    Free;
  end;
  end;

  ClassesItem.Free;
end;

procedure TJManualViewerMainForm.FormShortCut(var Msg: TWMKey;
  var Handled: Boolean);
begin
  if Msg.CharCode = VK_ESCAPE then begin
    Application.Terminate;
    Handled := true;
  end;
end;

//// c[o[

procedure TJManualViewerMainForm.ReloadPageClick(Sender: TObject);
begin
  Browser.Refresh;
end;

procedure TJManualViewerMainForm.ToTopPageClick(Sender: TObject);
begin
  GoPage('index.html');
end;

procedure TJManualViewerMainForm.SetHelpPathClick(Sender: TObject);
var Path : String;
begin
  Path := APIManualRoot;
  if SelectDirectory('JAVAIC}jÂtH_w肵܂B','' , Path) then begin
    APIManualRoot := IncludeTrailingPathDelimiter(Path);
    ReloadAPIList;
  end;
end;

//// API

procedure TJManualViewerMainForm.PackageListClick(Sender: TObject);
var
  Lines     : TStringList;
  i         : Integer;
  function LineCheck(Line: String): String;
  begin
    PickToken(Line, 'HREF="');
    Result := PickToken(Line, '"') + '|';
    if Pos('<I>', Line) > 0 then // C^[tF[X
      Result := Result + INTERFACE_ICON
    else if Pos('Exception', Line) > 0 then // O
      Result := Result + EXCEPTION_ICON
    else // ʃNX
      Result := Result + CLASS_ICON;
  end;

begin
  ClassesItem.Clear;
  Lines := TStringList.Create;
  try
    // ǂݍ
    if (PackageList.ItemIndex = 0) and
      FileExists(APIManualRoot + 'api\allclasses-frame.html') then
      Lines.LoadFromFile(APIManualRoot + 'api\allclasses-frame.html')
    else if FileExists(APIManualRoot + 'api\' +
        PackageNameToPath + '\package-frame.html') then
      Lines.LoadFromFile(APIManualRoot + 'api\' +
        PackageNameToPath + '\package-frame.html');
    // es[v
    for i := 0 to Lines.Count - 1 do
      // sɃN^O(NXy[Wւ̃NH)
      if Pos('<A', Lines[i]) > 0 then
        ClassesItem.Add(LineCheck(Lines[i]));
  finally
    Lines.Free;
  end;
  QuickSearchChange(nil);
end;

procedure TJManualViewerMainForm.QuickSearchChange(Sender: TObject);
var
  i : Integer;
  st: String;

begin
  if QuickSearch.Text = '' then
    ClassesList.Items.Assign(ClassesItem)
  else begin // 񂪓͂Ă
    st := LowerCase(QuickSearch.Text);
    ClassesList.Items.BeginUpdate;
    try
      ClassesList.Clear;
      for i := 0 to ClassesItem.Count - 1 do begin
        if AnsiContainsStr(LowerCase(PickConstTokenRev(ClassesItem[i], '/')), st) then
          ClassesList.AddItem(ClassesItem[i], nil);
      end;
    finally
      ClassesList.Items.EndUpdate;
    end;
  end;
end;

procedure TJManualViewerMainForm.QuickSearchKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_DOWN:
      with ClassesList do if CanFocus then SetFocus;
    VK_UP:
      with PackageList do if CanFocus then SetFocus;
  end;
end;

/// NX̕\

procedure TJManualViewerMainForm.ClassesListDblClick(Sender: TObject);
var str : String;
begin
  if ClassesList.ItemIndex > -1 then begin
    str := GetListCurrent(ClassesList);
    if PackageList.ItemIndex > 0 then
      GoPage('api\' + PackageNameToPath + PathDelim + PickToken(str, '|'))
    else
      GoPage('api\' + PickToken(str, '|'));
  end;
end;

procedure TJManualViewerMainForm.ClassesListKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key = VK_RETURN then ClassesListDblClick(nil);
end;

//// Xg{bNX̃I[i[h[

procedure TJManualViewerMainForm.ClassesListDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  DrawStr : String;
  TempText: String;
  IcoIndex: Integer;
const
  // 錾⊮o߂łBӂȂ
  Flags   : Integer = DT_VCENTER or DT_LEFT or DT_SINGLELINE or DT_NOPREFIX;
begin
  // 
  TempText:= ClassesList.Items[Index];
  DrawStr := PickToken(TempText, '|');
  DrawStr := PickTokenRev(DrawStr, '/');
  PickTokenRev(DrawStr, '.');
  // wi̕`
  ClassesList.Canvas.FillRect(Rect);
  // ACR̕`
  case TempText[1] of
    INTERFACE_ICON:
      IcoIndex := 1;
    EXCEPTION_ICON:
      IcoIndex := 2;
    CLASS_ICON:
      IcoIndex := 0;
  else
      IcoIndex := 0;
  end;
  ImageList.Draw(ClassesList.Canvas, Rect.Left,
    Rect.Top + CenterItem(Rect.Bottom - Rect.Top, ImageList.Height) ,
    IcoIndex);
  Inc(Rect.Left, ImageList.Width + 1);
  // ̕`
  DrawText(ClassesList.Canvas.Handle, PChar(DrawStr), -1, Rect, Flags);
  // tH[JXlp`̕`
  if odSelected in State then
    Canvas.DrawFocusRect(Classes.Rect(Point(Rect.Left - ImageList.Width - 1, Rect.Top), Rect.BottomRight));
end;

/// uEU

procedure TJManualViewerMainForm.URLComboClick(Sender: TObject);
begin
  Browser.Navigate(URLCombo.Text);
end;

procedure TJManualViewerMainForm.URLComboKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if Key = VK_RETURN then
    URLComboClick(nil);
end;

procedure TJManualViewerMainForm.BrowserStatusTextChange(Sender: TObject;
  const Text: WideString);
begin
  StatusBar.Panels[0].Text := Text;
end;

procedure TJManualViewerMainForm.BrowserTitleChange(Sender: TObject;
  const Text: WideString);
begin
  Caption := Format('%s(%s)', [Application.Title, Text]);
end;

procedure TJManualViewerMainForm.BrowserBeforeNavigate2(Sender: TObject;
  const pDisp: IDispatch; var URL, Flags, TargetFrameName, PostData,
  Headers: OleVariant; var Cancel: WordBool);
begin
  URLCombo.Text := URL
end;

procedure TJManualViewerMainForm.BrowserNavigateComplete2(Sender: TObject;
  const pDisp: IDispatch; var URL: OleVariant);
begin
  // 炵URLǉ
  if URLCombo.Items.IndexOf(URL) = -1 then
    URLCombo.AddItem(URL, nil);
  // 10𒴂A̕폜
  while URLCombo.Items.Count > 10 do URLCombo.Items.Delete(10);
end;

//// ֐

procedure TJManualViewerMainForm.GoPage(RelativeURL: String);
begin
  if APIManualRoot <> '' then
    Browser.Navigate(APIManualRoot + RelativeURL);
end;

procedure TJManualViewerMainForm.ReloadAPIList;
var frm : TForm;
begin
  frm := CreateMessageDialog('΂炭܂', mtCustom, []);
  try
    frm.Show;
    frm.Update;

    // ڂ̃NA
    PackageList.Clear;
    QuickSearch.Clear;
    // pbP[WXg̓ǂݍ
    if FileExists(APIManualRoot + 'api\package-list') then begin
      PackageList.Items.LoadFromFile(APIManualRoot + 'api\package-list');
      PackageList.Items.Insert(0, 'ׂẴNX');
      PackageList.ItemIndex := 0;
      PackageListClick(nil);
    end;
  finally
    frm.Release;
  end;
end;

function TJManualViewerMainForm.PackageNameToPath: String;
begin
  Result := StringReplace(PackageList.Text, '.', PathDelim, [rfReplaceAll]);
end;

end.

